from django.db import models

# Create your models here.
class BattleNetAccount(models.Model):
    user = models.ForeignKey('auth.User')
    bnet_name = models.CharField(max_length=1024)
    bnet_id = models.IntegerField()
    accountID = models.AutoField(primary_key=True)
    bnet_url = models.CharField(max_length=1024)
    
    @classmethod
    def create(cls, _user, _bnet_name, _bnet_id, _bnet_url):
        acct = cls(user=_user, bnet_name=_bnet_name, bnet_id=_bnet_id, bnet_url=_bnet_url)

        return acct
    
    def label_from_instance(self, obj):
        return "%s [%i]" % obj.bnet_name, obj.bnet_id

    def __str__(self):
        return str(self.bnet_name) + " [" + str(self.bnet_id) + "] " + str(self.accountID)

class BattleNetTeamAccount(models.Model):
    team_user = models.ForeignKey('auth.User')
    bnet_name = models.CharField(max_length=1024)
    bnet_id = models.IntegerField()
    accountID = models.AutoField(primary_key=True)
    bnet_url = models.CharField(max_length=1024)

    @classmethod
    def create(cls, _user, _bnet_name, _bnet_id, _bnet_url):
        acct = cls(team_user=_user, bnet_name=_bnet_name, bnet_id=_bnet_id, bnet_url=_bnet_url)

        return acct

    def __str__(self):
        return str(self.bnet_name) + " [" + str(self.bnet_id) + "] " + str(self.accountID)

class ArrangedTeam(models.Model):
    user = models.ForeignKey('auth.User')
    bnet_acct = models.ForeignKey(BattleNetAccount)
    teamName = models.CharField(max_length=128)
    teamType = models.CharField(max_length=5)
    members = models.ManyToManyField(BattleNetTeamAccount)
    teamID = models.AutoField(primary_key=True)

    def __str__(self):
        res = self.teamType + " team with "
        for member in self.members.all():
            res += str(member) + " "
            
        return res
