from django.shortcuts import render

from django.http import HttpResponse
from django.template.loader import render_to_string
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect
from django.template import RequestContext

from models import *

from inbox.models import *
from django.core.mail import send_mail
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.contrib.auth.decorators import login_required

def createAccount(request):
    from .forms import CreateAccountForm
    form = CreateAccountForm()

    content = render_to_string('create_new_account.html', { 'form': form }, context_instance = RequestContext(request))
    
    return HttpResponse(content)

def saveAccountToDatabase(request):
    if request.method == 'POST':
        from django.contrib.auth.models import User

        exists = User.objects.filter(email=request.POST['email'])
        if exists:
            content = render_to_string('account_created.html', {'msg': "Email already taken!"}, context_instance = RequestContext(request))
            return HttpResponse(content)
        else:
            user = User.objects.create_user(request.POST['email'], request.POST['email'], request.POST['password'])
            user.save()
            emailmsg = 'Username: ' + request.POST['email'] +"\n" + "Log in at http://replays.flyrev.org"
            send_mail('SC2Benchmarks Account Creation', emailmsg, 'replays@flyrev.org', [request.POST['email']], fail_silently=False)
            content = render_to_string('account_created.html', {'msg': "Account created! You can now log in (top right)."}, context_instance = RequestContext(request))
            return HttpResponse(content)   
    else:
        return HttpResponse("Please fill out the form on the previous page.")

def login(request):
    from .forms import LoginForm
    form = LoginForm()

    content = render_to_string('login.html', { 'form': form }, context_instance = RequestContext(request))
    return HttpResponse(content)

def processLogin(request):
    from django.shortcuts import redirect

    if request.method == 'POST':
        from django.contrib.auth import authenticate, login

        user = authenticate(username=request.POST['username'], password=request.POST['password'])

        if user is not None:
            if user.is_active:
                login(request, user)
                return redirect('/account/')
            else:
                return HttpResponse("The password is valid, but the account has been disabled!")
        else:
            return HttpResponse("Login absolutely incorrect.")
    else:
        return redirect('/account/')

from stats.models import *

def getStatsForUser(user):
    from collections import defaultdict

    stats = defaultdict(int)
    for bnet_acct in BattleNetAccount.objects.filter(user=user):
        for stat in WorkersCreated.objects.filter(bnet_id=bnet_acct):
            stats['workers_created'] += stat.workers_created
        for stat in WorkersKilled.objects.filter(bnet_id=bnet_acct):
            stats['workers_killed'] += stat.workers_killed
    return stats

def profile(request):
    from django.shortcuts import redirect

    if not request.user.is_authenticated():
        return redirect('/login/')
    else:
        from .models import BattleNetAccount
        
        accts = []

        accts_db = BattleNetAccount.objects.filter(user=request.user)
        for acct in accts_db:
            accts.append( {'name': acct.bnet_name, 'id': acct.accountID, 'url': acct.bnet_url } )

        sessions = []

        from buildpractice.models import PracticeSession, BenchmarkPracticeReplay
        sessions_db = PracticeSession.objects.filter(user=request.user)

        for sess in sessions_db:
            if sess.session_id.startswith('PUBLIC__'):
                continue

            score = [31]*10
            i=0
            for practiceReplay in BenchmarkPracticeReplay.objects.filter(session=sess).order_by('-ID')[:10]:
                late = practiceReplay.secondsLate
                if (late >= 0):
                    score[i] = min(score[i],late)
                i+=1

            total_score = sum(score)/10
            print score,total_score
            from buildpractice.views import secondsToLeague

            if i > 0:
                league = secondsToLeague(total_score)
            else:
                league = 'wood'

            score_text = None
            if i < 10:
                score_text = "Uploaded: " + str(i) + "/10"
            else:
                score_text = "Average seconds late: " + str(total_score)

            sessions.append({'build_name': sess.buildOrderName, 'session_id': sess.session_id, 'score': score_text, 'league': league})

        replays_in_inbox = 0
        analyzed_count = 0
        for rep in ReplayInInbox.objects.filter(user=request.user).all():
            replays_in_inbox+=1
            elements = []
            analyzed = False
            for element in ReplayAnalysisElement.objects.filter(replay=rep).all():
                elements.append(element)
                analyzed = True
            if analyzed: 
                analyzed_count+=1

        ratio = 0

        if replays_in_inbox == 0:
            analyzed_medal = 'gm_gray'
        elif analyzed_count == 0:
            analyzed_medal = 'bronze'
        else:
            ratio = float(analyzed_count) / float(replays_in_inbox)
            if ratio == 1:
                analyzed_medal = 'grandmaster'
            elif ratio >= 0.98:
                analyzed_medal = 'master'
            elif ratio >= 0.8:
                analyzed_medal = 'diamond'
            elif ratio >= 0.6:
                analyzed_medal = 'platinum'
            elif ratio >= 0.4:
                analyzed_medal = 'gold'
            elif ratio >= 0.2:
                analyzed_medal = 'silver'
            else:
                analyzed_medal = 'bronze'

        analyzed_percentage = int(ratio*100)

        stats = getStatsForUser(request.user)

        d = { 'PROFILE' : "", 'analyzed_percentage': analyzed_percentage,'analyzed_medal': analyzed_medal, 'new_replays': (replays_in_inbox-analyzed_count), 'analyzed': analyzed_count, 'total_replay\
s': replays_in_inbox, 'accounts': accts, 'practice_sessions': sessions}

        page_dict = dict(stats.items() + d.items())

        import math
        page_dict['achievement_progress'] = page_dict.get('workers_created', 0)
        page_dict['achievement_goal'] = 10**int(math.ceil(math.log(page_dict.get('workers_created', 10))/math.log(10)))
        page_dict['achievement_text'] = "Create " + str(page_dict.get('achievement_goal', 0)) + " workers"
        page_dict['achievement_percentage'] = int(100*page_dict.get('achievement_progress', 0) / page_dict.get('achievement_goal', 1))

        content = render_to_string('profile.html', page_dict, context_instance=RequestContext(request))
        return HttpResponse(content)

def getTeamMembers(teamID,request):
    for team in ArrangedTeam.objects.filter(user=request.user,teamID=teamID):
        teamMembers = []

        for member in team.members.all():
            teamMembers.append({'name': member.bnet_name, 'bnet_id': member.bnet_id})

    return teamMembers

@login_required
def PublicTeamSummary(request,team_id):
    from account.models import BattleNetAccount
    
    teamList = []
    for team in ArrangedTeam.objects.filter(teamID=int(team_id)):
        teamMembers = getTeamMembers(int(team_id),request)
        
        from stats.models import WinsAgainstLeague
        T=team
        winstats, c = WinsAgainstLeague.objects.get_or_create(team=T)
        wins = []
        wins.append(winstats.unknown_league_wins)
        wins.append(winstats.bronze_wins)
        wins.append(winstats.silver_wins)
        wins.append(winstats.gold_wins)
        wins.append(winstats.platinum_wins)
        wins.append(winstats.diamond_wins)
        wins.append(winstats.master_wins)
        wins.append(winstats.grandmaster_wins)

        defeated = sum(wins)
        victories = defeated / (len(teamMembers))

        teamType=str(len(teamMembers)) + "v" + str(len(teamMembers))

        teamList.append({'name': team.teamName, 'members': teamMembers, 'wins': wins, 'teamType': teamType, 'victories': victories, 'defeated': defeated })

    content = render_to_string('public_team_summary.html', {'team': teamList[0]}, context_instance=RequestContext(request))
    return HttpResponse(content)    

def BNetAccountSummary(request,user_id):
    print "UserID",user_id

    from stats.models import WorkersCreated
    from account.models import BattleNetAccount

    _bnet_id = BattleNetAccount.objects.get(user=request.user,accountID=int(user_id))

    workers = WorkersCreated.objects.filter(bnet_id=_bnet_id)
    
    obj = None
    for W in workers:
        obj = W

    workers_created = 0
    if obj != None:
        workers_created += obj.workers_created
    
    teamList = []
    for team in ArrangedTeam.objects.filter(user=request.user,bnet_acct=_bnet_id):
        teamMembers = getTeamMembers(team.teamID,request)
        
        from stats.models import WinsAgainstLeague
        T=team
        winstats, c = WinsAgainstLeague.objects.get_or_create(team=T)
        wins = []
        wins.append(winstats.unknown_league_wins)
        wins.append(winstats.bronze_wins)
        wins.append(winstats.silver_wins)
        wins.append(winstats.gold_wins)
        wins.append(winstats.platinum_wins)
        wins.append(winstats.diamond_wins)
        wins.append(winstats.master_wins)
        wins.append(winstats.grandmaster_wins)
        
        teamList.append({'name': team.teamName, 'members': teamMembers, 'wins': wins, 'id': team.teamID})

    content = render_to_string('profile_summary.html', {'name': _bnet_id.bnet_name, 'WORKERS_CREATED':workers_created, 'teams': teamList}, context_instance=RequestContext(request))
    return HttpResponse(content)

@login_required
def addBnetAccount(request):
    from .forms import AddBNetAccountForm

    form = AddBNetAccountForm()

    content = render_to_string('add_bnet_account.html', { 'form': form }, context_instance=RequestContext(request))

    return HttpResponse(content)

@login_required
def saveBnetAccount(request):
    profileID = None

    if request.POST.get('uid', -1) > 0:
        profileID = int(request.POST['uid'])

    if profileID is not None:
        from .models import BattleNetAccount
        from .models import BattleNetTeamAccount
        
        print "Here come the players",request.session['players']

        for player in request.session['players']:
            if player['uid'] == profileID:
                account_coupling,c = BattleNetAccount.objects.get_or_create(user=request.user, bnet_name=player['name'], bnet_id=player['uid'], bnet_url=player['url'])

                bnet_team_acct, c = BattleNetTeamAccount.objects.get_or_create(team_user=request.user, bnet_name=player['name'], bnet_id=player['uid'], bnet_url=player['url'])

                from account.models import ArrangedTeam
                from stats.models import WinsAgainstLeague
                obj,c = ArrangedTeam.objects.get_or_create(user=request.user, bnet_acct=account_coupling, teamName=player['name'], teamType="1v1")
                if c:
                    obj.members.add(bnet_team_acct)
                break

        return profile(request)
    else:
        return pickPlayerFromReplay(request)

    return HttpResponse("No UID given")

from django.conf import settings

@login_required
def pickPlayerFromReplay(request):
    from utilities.ReplayUploader import ReplayUploader
    from utilities.BenchmarkAnalyzer import BenchmarkAnalyzer
    from utilities.FrameTimeContinuum import FrameTimeContinuum

    from replayparsing import Replay
    import os

    if 'replayFile' in request.FILES:
        result = ReplayUploader().handle_uploaded_file_for_user(request,request.FILES['replayFile'])

        if result[0]:
            replay = result[1]
            players = []

            for player in replay.players:
                players.append( {'name': replay.getPlayerName(player), 'uid': replay.getBNetID(player), 'number': player, 'url': replay.getBNetURL(player) } )
                
            request.session['players'] = players
            print "Here come the players 1",request.session['players']

            content = render_to_string('select_player.html', { 'players': players }, context_instance=RequestContext(request))
            return HttpResponse(content)
        else:
            return HttpResponse("Something went wro")

    return HttpResponse("Please select replay")

@login_required
def addTeam(request):
    from .forms import AddTeamForm

    form = AddTeamForm()

    content = render_to_string('add_team.html', { 'form': form }, context_instance=RequestContext(request))

    return HttpResponse(content)

@login_required
def saveTeam(request):
    from replayparsing import Replay

    if request.POST.get('teamID', -1) > 0:
        teamID = int(request.POST['teamID'])

        replayHash = request.POST['replayhash']
        replay = Replay('/home/replays/uploaded/' + replayHash + ".SC2Replay")

        team = replay.getTeams()[teamID]

        from models import ArrangedTeam
        from models import BattleNetAccount

        teamStr = ""
        idStr = ""
        bnet_account = None

        for playerDict in team:
            teamStr += playerDict['name'] + ", "
            idStr += str(playerDict['uid']) + " "

            from django.core.exceptions import ObjectDoesNotExist
            try:
                if bnet_account == None:
                    bnet_account = BattleNetAccount.objects.get(user=request.user,bnet_id=playerDict['uid'])
            except ObjectDoesNotExist:
                bnet_account = None

        if bnet_account == None:
            return HttpResponse("None of your battle.net accounts were found in that team. Please add it.")

        teamStr = teamStr[:-2]
        matchup = str(len(team)) + "v" + str(len(team))
        arrangedTeam = ArrangedTeam.objects.create(user=request.user, bnet_acct=bnet_account, teamName=teamStr, teamType=matchup)

        for playerDict in team:
            account, c = BattleNetTeamAccount.objects.get_or_create(team_user=request.user, bnet_name=playerDict['name'], bnet_id = playerDict['uid'])
            arrangedTeam.members.add(account)
    
        arrangedTeam.save()

        return profile(request)
    else:
        return pickTeamFromReplay(request)

@login_required
def pickTeamFromReplay(request):
    from utilities.ReplayUploader import ReplayUploader
    from utilities.BenchmarkAnalyzer import BenchmarkAnalyzer
    from utilities.FrameTimeContinuum import FrameTimeContinuum

    from replayparsing import Replay
    import os

    if 'replayFile' in request.FILES:
        result = ReplayUploader().handle_uploaded_file(request.FILES['replayFile'])

        if result[0]:
            replay = result[1]
            teams = []

            for team in replay.getTeams():
                teamStr = ""
                idStr = ""
                for playerDict in replay.getTeams()[team]:
                    teamStr += playerDict['name'] + ", "
                    idStr += str(playerDict['uid']) + " "

                teams.append( {'team': teamStr, 'ID': team} )

            content = render_to_string('select_team.html', { 'teams': teams, 'replayhash': replay.hash }, context_instance=RequestContext(request))
            return HttpResponse(content)
        else:
            return HttpResponse("Something went wro")

    return HttpResponse("what trickery is .this")

