from django import forms
from utilities.UnitList import UnitList

from crispy_forms.helper import FormHelper
from crispy_forms.helper import Layout
from crispy_forms.layout import Submit
from crispy_forms.layout import HTML

from bootstrap_toolkit.widgets import BootstrapDateInput, BootstrapTextInput, BootstrapUneditableInput

import random

class NewPracticeSessionForm(forms.Form):
    builds = ['6 pool', '10 pool', '3 rax', 'CC first', '7 roach rush']

    buildOrderName = forms.CharField(label='Build order name',
                                     widget=BootstrapTextInput(
            attrs = {
               'placeholder': random.choice(builds),
               'required': 'true'
             }
                                     ))

    race = forms.ChoiceField(choices=[('Terran', 'Terran'), ('Protoss', 'Protoss'), ('Zerg', 'Zerg')], widget=forms.Select(attrs={'required': 'true'}))

class BenchmarkElementForm(forms.Form):

    def __init__(self, *args, **kwargs):
        race = kwargs.pop('race')
        super(BenchmarkElementForm, self).__init__(*args, **kwargs)

        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.layout = Layout(
            'time',
            'count',
            'unit',
            HTML('<br>'),
            Submit('submit', 'Add', css_class='btn-primary', ),
            )

        units = UnitList().units
        self.fields['unit'] = forms.ChoiceField(label='Unit type', choices=units[race], widget=forms.Select(attrs={'required': 'true'}))

    time = forms.CharField(
         widget=BootstrapTextInput(
        attrs =  {
            'placeholder': '05:00',
            'required': 'true'
            }))

    count = forms.CharField(
            widget=BootstrapTextInput(
                attrs = {
                    'placeholder': '5',
                    'required': 'true'
                    }))

class BenchmarkBuildingElementForm(forms.Form):

    def __init__(self, *args, **kwargs):
        race = kwargs.pop('race')
        print "BenchmarkBuildingElementForm"
        super(BenchmarkBuildingElementForm, self).__init__(*args, **kwargs)

        buildings = UnitList().buildings

        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.layout = Layout(
            'time',
            'count',
            'building',
            HTML('<br>'),
            Submit('submit', 'Add', css_class='btn-primary', ),
            )

        self.fields['building'] = forms.ChoiceField(label='Building', choices=buildings[race], widget=forms.Select(attrs={'required': 'true'}))



    time = forms.CharField(
        widget=BootstrapTextInput(
        attrs =  {
            'placeholder': '06:00',
            'required': 'true'
            }))

    count = forms.CharField(
            widget=BootstrapTextInput(
                attrs = {
                    'placeholder': '5',
                    'required': 'true'
                    }))


class BenchmarkUpgradeForm(forms.Form):

    def __init__(self, *args, **kwargs):
        race = kwargs.pop('race')
        super(BenchmarkUpgradeForm, self).__init__(*args, **kwargs)

        upgrades = UnitList().upgrades
        self.fields['upgrade'] = forms.ChoiceField(label='Upgrade', choices=upgrades[race], widget=forms.Select(attrs={'required': 'true'}))

        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.layout = Layout(
            'time',
            'upgrade',
            HTML('<br>'),
            Submit('submit', 'Add', css_class='btn-primary', ),
            )

    time = forms.CharField(
         widget=BootstrapTextInput(
        attrs =  {
            'placeholder': '10:00',
            'required': 'true'
            }))

class PracticeReplay(forms.Form):
    replayFile = forms.FileField()


class ShareBenchmarkForm(forms.Form):
    def __init__(self, *args, **kwargs):
        bo_name = kwargs.pop('bo_name')
        race = kwargs.pop('race')
        super(ShareBenchmarkForm, self).__init__(*args, **kwargs)


        self.fields['buildOrderName'] = forms.CharField(label='Build order name',
                                     widget=BootstrapTextInput(
                attrs = {
                    'value': bo_name,
                    'required': 'true'
                    }
                ))

        self.fields['matchup'] = forms.ChoiceField(
            choices=[(race + ' vs Terran', race + ' vs Terran'), 
                      (race + ' vs Protoss', race + ' vs Protoss'), 
                      (race + ' vs Zerg', race + ' vs Zerg'),
                      (race + ' vs Any', race + ' vs Any')], widget=forms.Select(attrs={'required': 'true'}))
            
        self.helper = FormHelper()
        self.helper.layout = Layout(
            'buildOrderName',
            'matchup',
#            'race',
            'url',
            'info',
            HTML('<br>'),
            Submit('submit', 'Publish', css_class='btn-primary', ),
            )



    url = forms.CharField(label='URL to build order description, if any')
    info = forms.CharField(label='Any other info about this build order?', widget=forms.Textarea)

