from django.db import models
from django.utils.encoding import smart_str, smart_unicode

class PracticeSession(models.Model):
    from account.models import BattleNetAccount

    user = models.ForeignKey('auth.User')
    race = models.CharField(max_length='20')

    buildOrderName = models.CharField(max_length=200)
    session_id = models.CharField(max_length=30)

    @classmethod
    def create(cls,_user,name,session):
        newInstance = cls(user=_user, buildOrderName=name, session_id=session)
        return newInstance

    def __str__(self):
        return smart_str(self.user.username) + ": " + smart_str(self.buildOrderName) + " [" + smart_str(self.session_id) + "]"

class PublicPracticeSession(models.Model):
    session = models.ForeignKey(PracticeSession)
    matchup = models.CharField(max_length=1024)
    link = models.CharField(max_length=1024)
    description = models.CharField(max_length=102400)

    def __str__(self):
        return smart_str(self.session) + " PublicPracticeSession.id = " + str(self.id)

class BenchmarkElement(models.Model):
    session = models.ForeignKey(PracticeSession)
    unit = models.CharField(max_length = 50)
    time = models.CharField(max_length = 10)
    count = models.IntegerField(default=0)

    @classmethod
    def create(cls,_session, _unit, _time, _count):
        newInstance = cls(session=_session, unit=_unit, time=_time, count=_count)
        return newInstance

    def __str__(self):
        return smart_str(self.session.user) + " " + smart_str(self.session.buildOrderName) + " " + smart_str(self.count) + " x " + smart_str(self.unit) + " @ " + smart_str(self.time) + " " + smart_str(self.session)

class BenchmarkUpgradeElement(models.Model):
    session = models.ForeignKey(PracticeSession)
    upgrade = models.CharField(max_length = 50)
    time = models.CharField(max_length = 10)

    @classmethod
    def create(cls,_session, _upgrade, _time):
        newInstance = cls(session=_session, upgrade=_upgrade, time=_time)
        return newInstance

    def __str__(self):
        return smart_str(self.session.user) + " " + smart_str(self.session.buildOrderName) + " " + smart_str(self.upgrade) + " @ " + smart_str(self.time) + " " + smart_str(self.session)

class BenchmarkBuildingElement(models.Model):
    session = models.ForeignKey(PracticeSession)
    building = models.CharField(max_length = 50)
    time = models.CharField(max_length = 10)
    count = models.IntegerField(default=0)

    @classmethod
    def create(cls,_session, _building, _time, _count):
        newInstance = cls(session=_session, building=_building, time=_time, count=_count)
        return newInstance

    def __str__(self):
        return smart_str(self.session.user) + " " + smart_str(self.session.buildOrderName) + " " + str(self.count) + " x " + smart_str(self.building) + " @ " + smart_str(self.time) + " " + str(self.session)

class BenchmarkPracticeReplay(models.Model):
    from analysis.models import Replay

    session = models.ForeignKey(PracticeSession)
    replayhash = models.ForeignKey(Replay)
    secondsLate = models.IntegerField(default=0)
    ID = models.AutoField(primary_key=True)

    @classmethod
    def create(cls,_session,_replayhash,_secondsLate):
        newInstance = cls(session=_session, replayhash=_replayhash, secondsLate=_secondsLate)
        return newInstance
    
    def __str__(self):
        return smart_str(str(self.session.user) + " " + self.session.buildOrderName + " " + str(self.secondsLate) + " seconds late" + " " + str(self.session))
