from django.shortcuts import render
from django.http import HttpResponse
from django.template import RequestContext
from django.template.loader import render_to_string
from django.shortcuts import render_to_response
from .forms import *
from .models import *
from django.shortcuts import redirect
from collections import defaultdict

from django.contrib.auth.decorators import login_required

def show_all_emails(request):
    from django.contrib.auth.models import User
    string = ""
    for user in User.objects.all():
        string += user.email + ", "
    return HttpResponse(string)

def secondsToLeague(seconds):
    league = None

    if seconds < 0:
        league = 'wood'
    elif seconds <= 5:
        league = 'grandmaster'
    elif seconds <= 10:
        league = 'master'
    elif seconds <= 15:
        league = 'diamond'
    elif seconds <= 20:
        league = 'platinum'
    elif seconds <= 25:
        league = 'gold'
    elif seconds <= 30:
        league = 'silver'
    else:
        league = 'bronze'

    return league

def listPublicBenchmarks(request):
    lst = defaultdict(list)
    for obj in PublicPracticeSession.objects.all():
        lst[obj.matchup].append(obj)

    lst = dict(lst)

    keys_sorted = sorted(lst.keys())

    content = render_to_string('list_of_public_benchmarks.html', {'lst': lst, 'keys_sorted': keys_sorted},
                              context_instance=RequestContext(request))

    return HttpResponse(content)    

def sharePracticeSession(request, sessionID):
    if request.method == 'POST':
        session = PracticeSession.objects.get(user=request.user, session_id=sessionID)
        
        # Make copy so that the author cannot edit or delete it afterwards and mess with everyone.
        # Also, the user should be free to remove it from their main dashboard, after all.

        publicSession = PracticeSession(user=request.user, race=session.race, session_id='PUBLIC__', buildOrderName=request.POST.get('buildOrderName', session.buildOrderName))
        publicSession.save()

        for b in BenchmarkElement.objects.filter(session=session).all():
            obj = BenchmarkElement(session=publicSession, unit=b.unit, time=b.time, count=b.count)
            obj.save()

        for b in BenchmarkUpgradeElement.objects.filter(session=session):
            BenchmarkUpgradeElement(session=publicSession, upgrade=b.upgrade, time=b.time).save()

        for b in BenchmarkBuildingElement.objects.filter(session=session):
            BenchmarkBuildingElement(session=publicSession, building=b.building, time=b.time, count=b.count).save()

        obj = PublicPracticeSession(session=publicSession, link=request.POST.get('url', ''), description=request.POST.get('info', ''), matchup=request.POST['matchup'])
        obj.save()

        # Silly ID hack, just because I like IDs in my __str__
        publicSession.session_id += "_" + str(obj.id)
        publicSession.save()

        request.session['msg'] = "This is what the public build order page looks like. Copy the address in the address bar and send it to all your friends. Or <a href='/practice/upload_replays/8utwkor0p4khvq1ul3g0ku9r/'>go back to your own practice session</a>."
        return redirect('/practice/builds/' + str(obj.id) + '/')
        
    privateSession = PracticeSession.objects.get(user=request.user, session_id=sessionID)
    form = ShareBenchmarkForm(bo_name=privateSession.buildOrderName, race=privateSession.race)

    unit_benchmarks = get_unit_benchmarks(request,sessionID)
    upgrade_benchmarks = get_upgrade_benchmarks(request,sessionID)
    building_benchmarks = get_building_benchmarks(request,sessionID)

    render_dict = {}
    render_dict['unit_benchmarks'] = unit_benchmarks
    render_dict['upgrade_benchmarks'] = upgrade_benchmarks
    render_dict['building_benchmarks'] = building_benchmarks
    render_dict['form'] = form
    render_dict['SESSION_ID'] = sessionID
    
    content = render_to_string('share_benchmarks.html', render_dict,
                              context_instance=RequestContext(request))

    return HttpResponse(content)

def deletePracticeSession(request, sessionID):
    from .models import PracticeSession

    if not request.user.is_authenticated():
        return

    PracticeSession.objects.filter(user=request.user, session_id=sessionID).delete()
    
    return HttpResponse("Deleted")

def createRandomID():
    N = 24
    import random
    import string

    return ''.join(random.choice(string.ascii_lowercase + string.digits) for x in range(N))

def createNewPracticeSession(request):
    if not request.user.is_authenticated():
        return

    from .forms import NewPracticeSessionForm

    form = NewPracticeSessionForm()

    random_id = createRandomID()
    content = render_to_string('new_practice_session.html', {'form': form, 'SESSION_ID': random_id}, 
                              context_instance=RequestContext(request))

    return HttpResponse(content)

def get_unit_benchmarks(request,sessionID):
    result = []

    print "get_unit_benchmarks for " + str(sessionID)
    practice_session = PracticeSession.objects.get(user=request.user, session_id=sessionID)
    benchmarkElements = BenchmarkElement.objects.filter(session=practice_session)

    for benchmark in benchmarkElements:
        result.append({'count': benchmark.count, 'name': benchmark.unit, 'unit': benchmark.unit, 'humanName':  UnitList().getHumanName(benchmark.unit), 'time': str(benchmark.time), 'id': benchmark.id})

    return result

def get_building_benchmarks(request,sessionID):
    result = []

    practice_session = PracticeSession.objects.get(user=request.user, session_id=sessionID)
    benchmarkElements = BenchmarkBuildingElement.objects.filter(session=practice_session)

    for benchmark in benchmarkElements:
        result.append({'count': benchmark.count, 'name': benchmark.building, 'building': benchmark.building, 'humanName':  UnitList().getHumanName(benchmark.building), 'time': str(benchmark.time), 'id': benchmark.id})

    return result

def get_upgrade_benchmarks(request,sessionID):
    result = []

    lst = PracticeSession.objects.filter(user=request.user, session_id=sessionID)
    practice_session = None

    for E in lst:
        practice_session = E
        break

    benchmarkElements = BenchmarkUpgradeElement.objects.filter(session=practice_session)

    for benchmark in benchmarkElements:
        result.append({'name': benchmark.upgrade, 'upgrade': benchmark.upgrade, 'humanName': UnitList().getHumanName(benchmark.upgrade), 'time': str(benchmark.time), 'id': benchmark.id})
        
    return result

def get_public_unit_benchmarks(request,sessionID):
    result = []

    print "get_unit_benchmarks for " + str(sessionID)
    practice_session = PracticeSession.objects.get(session_id=sessionID)
    benchmarkElements = BenchmarkElement.objects.filter(session=practice_session)

    for benchmark in benchmarkElements:
        result.append({'count': benchmark.count, 'name': benchmark.unit, 'unit': benchmark.unit, 'humanName':  UnitList().getHumanName(benchmark.unit), 'time': str(benchmark.time), 'id': benchmark.id})

    return result

def get_public_building_benchmarks(request,sessionID):
    result = []

    practice_session = PracticeSession.objects.get(session_id=sessionID)
    benchmarkElements = BenchmarkBuildingElement.objects.filter(session=practice_session)

    for benchmark in benchmarkElements:
        result.append({'count': benchmark.count, 'name': benchmark.building, 'building': benchmark.building, 'humanName':  UnitList().getHumanName(benchmark.building), 'time': str(benchmark.time), 'id': benchmark.id})

    return result

def get_public_upgrade_benchmarks(request,sessionID):
    result = []

    lst = PracticeSession.objects.filter(session_id=sessionID)
    practice_session = None

    for E in lst:
        practice_session = E
        break

    benchmarkElements = BenchmarkUpgradeElement.objects.filter(session=practice_session)

    for benchmark in benchmarkElements:
        result.append({'name': benchmark.upgrade, 'upgrade': benchmark.upgrade, 'humanName': UnitList().getHumanName(benchmark.upgrade), 'time': str(benchmark.time), 'id': benchmark.id})
        
    return result
    
def valid_time(string):
    splitted = string.split(':')
    if len(splitted) != 2:
        return False
    try:
        minutes = int(splitted[0])
        seconds = int(splitted[1])
    except ValueError:
        return False

    return True

def deleteElementFromSession(request,sessionID,elementID):
    session = PracticeSession.objects.get(user=request.user, session_id=sessionID)
    BenchmarkElement.objects.filter(session=session, id=elementID).delete()

    return HttpResponse("Benchmark element deleted")

def deleteBuildingFromSession(request,sessionID,elementID):
    session = PracticeSession.objects.get(user=request.user, session_id=sessionID)
    BenchmarkBuildingElement.objects.filter(session=session, id=elementID).delete()

    return HttpResponse("Benchmark building element deleted")

def deleteUpgradeElementFromSession(request,sessionID,elementID):
    session = PracticeSession.objects.get(user=request.user, session_id=sessionID)
    BenchmarkUpgradeElement.objects.filter(session=session, id=elementID).delete()

    return HttpResponse("Benchmark upgrade element deleted")

def addElementsToSession(request,sessionID):
    from .models import PracticeSession
    from .models import BenchmarkElement

    if 'buildOrderName' in request.POST:
        request.session['buildOrderName'] = request.POST['buildOrderName']
        practice_session, created = PracticeSession.objects.get_or_create(user=request.user, buildOrderName=request.session['buildOrderName'], session_id=sessionID)
    else:
        practice_session = PracticeSession.objects.get(user=request.user, session_id=sessionID)
        created=False

    if created:
        practice_session.race = request.POST['race']
        practice_session.save()

    race = practice_session.race

    form = BenchmarkElementForm(race=race)
    upgradeForm = BenchmarkUpgradeForm(race=race)
    buildingForm = BenchmarkBuildingElementForm(race=race)

    error = ""

    if 'unit' in request.POST:
        sessionObjects = PracticeSession.objects.filter(session_id=sessionID)

        if not valid_time(request.POST['time']):
            error = "Invalid time format. Use minutes:seconds, for example 10:00."
            
        else:
            element = BenchmarkElement(session=practice_session, unit=request.POST['unit'], time=request.POST['time'], count=int(request.POST['count']))
            element.save()

    if 'upgrade' in request.POST:
        sessionObjects = PracticeSession.objects.filter(session_id=sessionID)

        if not valid_time(request.POST['time']):
            error = "Invalid time format. Use minutes:seconds, for example 10:00."        

        else:
            element = BenchmarkUpgradeElement(session=practice_session, upgrade=request.POST['upgrade'], time=request.POST['time'])
            element.save()


    if 'building' in request.POST:
        sessionObjects = PracticeSession.objects.filter(session_id=sessionID)

        if not valid_time(request.POST['time']):
            error = "Invalid time format. Use minutes:seconds, for example 10:00."        

        else:
            element = BenchmarkBuildingElement(session=practice_session, building=request.POST['building'], time=request.POST['time'], count=int(request.POST['count']))
            element.save()

    unit_benchmarks = get_unit_benchmarks(request,sessionID)
    upgrade_benchmarks = get_upgrade_benchmarks(request,sessionID)
    building_benchmarks = get_building_benchmarks(request,sessionID)

    practiceButtonDisabled = "disabled"
    if len(unit_benchmarks) + len(upgrade_benchmarks) + len(building_benchmarks) > 0:
        practiceButtonDisabled = "enabled"

    render_dict = {'error': error, 'practiceButtonDisabled': practiceButtonDisabled, 'SESSION_ID': sessionID, 'BO_NAME': practice_session.buildOrderName}
    render_dict['unit_benchmarks'] = unit_benchmarks
    render_dict['upgrade_benchmarks'] = upgrade_benchmarks
    render_dict['building_benchmarks'] = building_benchmarks

    render_dict['form'] = form
    render_dict['upgradeForm'] = upgradeForm
    render_dict['buildingForm'] = buildingForm

    content = render_to_string('add_elements.html', render_dict, context_instance=RequestContext(request))

    return HttpResponse(content)

@login_required
def publicBenchmarks(request, ID):
    public_practice_session = PublicPracticeSession.objects.get(id=int(ID))

    sessionID=public_practice_session.session.session_id

    benchmarks = get_public_unit_benchmarks(request, sessionID)
    upgradeBenchmarks = get_public_upgrade_benchmarks(request, sessionID)
    building_benchmarks = get_public_building_benchmarks(request,sessionID)

    render_dict = {}
    
    render_dict['unit_benchmarks'] = benchmarks
    render_dict['upgrade_benchmarks'] = upgradeBenchmarks
    render_dict['building_benchmarks'] = building_benchmarks
    render_dict['BUILD_ORDER'] = public_practice_session.session.buildOrderName
    render_dict['benchmark_id'] = public_practice_session.id
    render_dict['url'] = public_practice_session.link
    render_dict['info'] = public_practice_session.description

    render_dict['msg'] = request.session.get('msg', '')

    request.session['msg'] = ''


    content = render_to_string('public_benchmarks.html', render_dict, context_instance=RequestContext(request))
    return HttpResponse(content)

@login_required
def startPracticeSession(request, ID):
    practice_session = PublicPracticeSession.objects.get(id=int(ID))
    sessionID = practice_session.session.session_id

    benchmarks = get_public_unit_benchmarks(request, sessionID)
    upgradeBenchmarks = get_public_upgrade_benchmarks(request, sessionID)
    building_benchmarks = get_public_building_benchmarks(request,sessionID)

    render_dict = {}
    
    render_dict['unit_benchmarks'] = benchmarks
    render_dict['upgrade_benchmarks'] = upgradeBenchmarks
    render_dict['building_benchmarks'] = building_benchmarks
    render_dict['BUILD_ORDER'] = practice_session.session.buildOrderName

    sid = createRandomID()
    importedSession = PracticeSession(user=request.user, race=practice_session.session.race, session_id=sid, buildOrderName=practice_session.session.buildOrderName)
    importedSession.save()
        
    for b in BenchmarkElement.objects.filter(session=practice_session.session).all():
        obj = BenchmarkElement(session=importedSession, unit=b.unit, time=b.time, count=b.count)
        obj.save()

    for b in BenchmarkUpgradeElement.objects.filter(session=practice_session.session):
        BenchmarkUpgradeElement(session=importedSession, upgrade=b.upgrade, time=b.time).save()

    for b in BenchmarkBuildingElement.objects.filter(session=practice_session.session):
        BenchmarkBuildingElement(session=importedSession, building=b.building, time=b.time, count=b.count).save()

    return redirect('/practice/upload_replays/' + sid + '/')

def uploadReplaysToSession(request,sessionID):
    from .models import PracticeSession
    from .models import BenchmarkElement

    if 'buildOrderName' in request.POST:
        request.session['buildOrderName'] = request.POST['buildOrderName']
        practice_session, created = PracticeSession.objects.get_or_create(user=request.user, buildOrderName=request.session['buildOrderName'], session_id=sessionID)
    else:
        practice_session = PracticeSession.objects.get(user=request.user, session_id=sessionID)
        created=False

    import analysis.models

    from replayparsing import Replay
    import os

    from utilities.ReplayUploader import ReplayUploader
    from utilities.BenchmarkAnalyzer import BenchmarkAnalyzer
    from utilities.FrameTimeContinuum import FrameTimeContinuum

    benchmarks = get_unit_benchmarks(request, sessionID)
    upgradeBenchmarks = get_upgrade_benchmarks(request, sessionID)
    building_benchmarks = get_building_benchmarks(request,sessionID)

    benchmarks_satisfied = ""
    
    tmp = request.session.get('replays', [])
    for replayFile in request.FILES.getlist('replayFiles'):
        result = ReplayUploader().handle_uploaded_file_for_user(request,replayFile)

        from utilities.BNet import BNet
        bnet_ids = BNet().getStoredBnetIDs(request)

        if result[0]:
            replay = result[1]
            benchmarks_satisfied = "Replay: " + replayFile.name + ":\n"

            for player in replay.players:
                if replay.getBNetID(player) not in bnet_ids:
                    continue
                    
                max_late = 0
    
                all_benchmarks_satisfied = True
                try:
                    rep,created = analysis.models.Replay.objects.get_or_create(hashValue=result[1].hash)
                except:
                    rep=analysis.models.Replay.objects.filter(hashValue=result[1].hash).latest('id')
                    created=False
                
                benchmarkResult = BenchmarkAnalyzer().checkBenchmarkByPlayerNumber(replay,player,benchmarks)
                for element in range(len(benchmarkResult)):
                    benchmarks_satisfied += replay.getPlayerName(player)
                    bench = str(benchmarks[element]['count']) + " x " + UnitList().getHumanName(benchmarks[element]['unit']) + " @ " + benchmarks[element]['time']
                    if benchmarkResult[element] > 0:
                        benchmarks_satisfied += " reached benchmark " + str((element+1)) + " (" + bench + ") at time " + str(FrameTimeContinuum().frameToTime(benchmarkResult[element]))
                        diff = int(benchmarkResult[element])-FrameTimeContinuum().timeToFrame(benchmarks[element]['time'])
                        if diff > 0:
                            secs = FrameTimeContinuum().frameToSeconds(diff)
                            max_late = max(secs,max_late)
                            benchmarks_satisfied += ", meaning " + str(secs) +  " seconds late"
                    else:
                        all_benchmarks_satisfied=False
                        benchmarks_satisfied += " never reached benchmark " + str((element+1)) + " (" + bench + ")"
                    benchmarks_satisfied += " \n "

                buildingResult = BenchmarkAnalyzer().checkBuildingBenchmarkByPlayerNumber(replay,player,building_benchmarks)
                for element in range(len(buildingResult)):
                    benchmarks_satisfied += replay.getPlayerName(player)
                    bench = str(building_benchmarks[element]['count']) + " x " + UnitList().getHumanName(building_benchmarks[element]['building']) + " @ " + building_benchmarks[element]['time']
                    if buildingResult[element] > 0:
                        benchmarks_satisfied += " reached benchmark " + str((element+1)) + " (" + bench + ") at time " + str(FrameTimeContinuum().frameToTime(buildingResult[element]))
                        diff = int(buildingResult[element])-FrameTimeContinuum().timeToFrame(building_benchmarks[element]['time'])
                        if diff > 0:
                            secs = FrameTimeContinuum().frameToSeconds(diff)
                            max_late = max(secs,max_late)
                            benchmarks_satisfied += ", meaning " + str(secs) +  " seconds late"
                    else:
                        all_benchmarks_satisfied=False
                        benchmarks_satisfied += " never reached benchmark " + str((element+1)) + " (" + bench + ")"
                    benchmarks_satisfied += " \n "


                upgradeBenchmarkResult = BenchmarkAnalyzer().checkUpgradeBenchmarkByPlayerNumber(replay,player,upgradeBenchmarks)
                for element in range(len(upgradeBenchmarkResult)):
                    benchmarks_satisfied += replay.getPlayerName(player)
                    bench = UnitList().getHumanName(upgradeBenchmarks[element]['upgrade']) + " @ " + upgradeBenchmarks[element]['time']
                    if upgradeBenchmarkResult[element] > 0:
                        benchmarks_satisfied += " reached benchmark " + str((element+1)) + " (" + bench + ") at time " + str(FrameTimeContinuum().frameToTime(upgradeBenchmarkResult[element]))
                        diff = int(upgradeBenchmarkResult[element])-FrameTimeContinuum().timeToFrame(upgradeBenchmarks[element]['time'])
                        if diff > 0:
                            secs = FrameTimeContinuum().frameToSeconds(diff)
                            max_late = max(secs,max_late)
                            benchmarks_satisfied += ", meaning " + str(secs) +  " seconds late"
                    else:
                        all_benchmarks_satisfied=False
                        benchmarks_satisfied += " never reached benchmark " + str((element+1)) + " (" + bench + ")"
                    benchmarks_satisfied += " \n "

                if not all_benchmarks_satisfied:
                    max_late = -1

                benchmarkReplay,c = BenchmarkPracticeReplay.objects.get_or_create(session=practice_session, replayhash=rep, secondsLate=max_late)
                print "benchmarkReplay created?",str(c)
        else:
            benchmarks_satisfied = 'Parse error'

        tmp.append((replayFile.name, benchmarks_satisfied))
        request.session['replays'] = tmp

    results_so_far = []

    replayString = "Replays:\n"
    for replayResult in BenchmarkPracticeReplay.objects.select_related().filter(session=practice_session).order_by('-ID')[:10]:
        replayString += replayResult.replayhash.hashValue + ": " + str(replayResult.secondsLate) + "\n"
        
        secs = replayResult.secondsLate
        league = secondsToLeague(secs)

        if (secs < 0):
            score = "Benchmarks never reached"
        elif secs == 0:
            score = "Perfect!"
        else:
            score = str(secs) + " seconds late"

        results_so_far.append({'mapName': replayResult.replayhash.hashValue, 'league': league, 'score': score})

    form = PracticeReplay()

    render_dict = {'SESSION_ID': sessionID, 'results': results_so_far}
    render_dict['unit_benchmarks'] = benchmarks
    render_dict['upgrade_benchmarks'] = upgradeBenchmarks
    render_dict['building_benchmarks'] = building_benchmarks
    render_dict['form'] = form
    render_dict['report'] = benchmarks_satisfied

    render_dict['BUILD_ORDER'] = "Unnamed build order"
    if practice_session != None:
        render_dict['BUILD_ORDER'] = practice_session.buildOrderName        

    content = render_to_string('upload_replays.html', render_dict, context_instance=RequestContext(request))
    return HttpResponse(content)

