from django.conf.urls import patterns, include, url

from django.contrib import admin
admin.autodiscover()

from analysis import views
from buildpractice import views as bviews
from account import views as account_views
from achievements import views as achievement_views
from inbox import views as inbox_views
from info import views as infoView

import django.contrib.auth.views as auth_views

def test_bootstrap(request):
    from django.http import HttpResponse
    from django.template.loader import render_to_string

    content = render_to_string('bootstrap_test.html', {})

    return HttpResponse(content)

def indexPage(request):
    from django.shortcuts import redirect

    if request.user.is_authenticated():
        return redirect('/account/')

    from django.http import HttpResponse
    from django.template.loader import render_to_string
    from django.http import HttpResponseRedirect
    from django.template import RequestContext

    from django.template.loader import render_to_string
    from django.shortcuts import render_to_response

    content = render_to_string('index.html', {}, context_instance = RequestContext(request))

    return HttpResponse(content)

def logout(request):
    from django.contrib.auth import logout
    logout(request)
    for sesskey in request.session.keys():
        del request.session[sesskey]

    return indexPage(request)

def settingsPage(request):
    from django.http import HttpResponse
    from django.template.loader import render_to_string
    from django.shortcuts import render_to_response
    from django.template import RequestContext
    from account.models import BattleNetAccount
    
    accts = []

    accts_db = BattleNetAccount.objects.filter(user=request.user)
    for acct in accts_db:
        accts.append( {'name': acct.bnet_name, 'id': acct.accountID, 'url': acct.bnet_url } )

    content = render_to_string('settings.html', {'accounts': accts}, context_instance = RequestContext(request))

    return HttpResponse(content)

def deleteBnetAccount(request,account_id):
    from django.http import HttpResponse
    from django.template.loader import render_to_string
    from django.shortcuts import render_to_response
    from django.template import RequestContext
    from account.models import BattleNetAccount
    if not request.user.is_authenticated():
        return

    BattleNetAccount.objects.filter(user=request.user, accountID=int(account_id)).delete()
    
    return HttpResponse("Deleted")

urlpatterns = patterns('',
    url(r'^analysis/', include('analysis.urls')),
    url(r'^upload', views.uploadReplay),
    url(r'^download_replay/(\d+)/(.*)/$', views.downloadReplay),
    url(r'^analyze_replay/', include('analysis.urls')),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^practice', include('buildpractice.urls')),
    url(r'^achievements/earned', achievement_views.achievementsEarnedList),
    url(r'^account/bnet/add', account_views.addBnetAccount),
    url(r'^account/bnet/save', account_views.saveBnetAccount),
    url(r'^account/team/add', account_views.addTeam),
    url(r'^account/team/save', account_views.saveTeam),
    url(r'^account/create/confirm', account_views.saveAccountToDatabase),
    url(r'^account/create/', account_views.createAccount),
    url(r'^account/login/process', account_views.processLogin),
    url(r'^account/login', account_views.login),
    url(r'^account/profile', account_views.profile),
    url(r'^account/(\d+)', account_views.BNetAccountSummary),
    url(r'^account/delete/(\d+)/', deleteBnetAccount),

    url(r'^password/change/$',
        auth_views.password_change,
        name='auth_password_change'),
    url(r'^password/change/done/$',
        auth_views.password_change_done,
        name='password_change_done'),
    url(r'^password/reset/$',
        auth_views.password_reset,
        name='auth_password_reset'),
    url(r'^password/reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        auth_views.password_reset_confirm,
        name='password_reset_confirm'),
    url(r'^password/reset/complete/$',
        auth_views.password_reset_complete,
        name='password_reset_complete'),
    url(r'^password/reset/done/$',
        auth_views.password_reset_done,
        name='password_reset_done'),


    url(r'^inbox/upload', inbox_views.uploadReplaysToInbox),
    url(r'^inbox/analyze', inbox_views.viewInbox),
    url(r'^analyze/add_elements/(.*)/$', inbox_views.add_analysis_element),
    url(r'^analyze/(.*)', inbox_views.analyzeReplay),

    url(r'^team/(\d+)', account_views.PublicTeamSummary),
    url(r'^account', account_views.profile),
    url(r'^test/', test_bootstrap),
    url(r'^logout', logout),
    url(r'^settings', settingsPage),

    url(r'^info/benchmarks/', infoView.benchmarksPage),
    url(r'^info/production/', infoView.productionPage),
    url(r'^info/statistics/', infoView.statisticsPage),

    url(r'^emails', bviews.show_all_emails),
                      
    url(r'^', indexPage),
)
