from collections import defaultdict

class BenchmarkAnalyzer:
    def __init__(self):
        pass

    def getGatewayUnits(self):
        return ['Zealot', 'Stalker', 'Sentry', 'HighTemplar', 'DarkTemplar']

    def checkBenchmarkByPlayerNumber(self,replay,playerNumber,benchmarks):
        result = []

        for benchmark in benchmarks:
            count = []
            arr = benchmark['time'].split(':')
            if len(arr) == 1:
                frames = int(arr[0])*60*16
            else:
                frames = (int(arr[0])*60+int(arr[1]))*16
            unit = benchmark['unit']
            count = int(benchmark['count'])
            warpInUnits = {}
            owned = defaultdict(lambda: False)

            for event in replay.trackerEvents:
                if event.get('_event', '') == 'NNet.Replay.Tracker.SUnitBornEvent' and event['m_controlPlayerId'] == playerNumber:
                    born_unit = event['m_unitTypeName']
                    if unit == born_unit:
                        count -= 1
                        if count == 0:
                            result.append(str(event.get('_gameloop', -1)))
                            break
                    owned[event['m_unitTagIndex']] = True

                # Warpins, beginning
                elif event.get('_event', '') == 'NNet.Replay.Tracker.SUnitInitEvent' and event['m_controlPlayerId'] == playerNumber:
                    if event['m_unitTypeName'] in self.getGatewayUnits() and unit == event['m_unitTypeName']:
                        warpInUnits[event['m_unitTagIndex']] = event['m_unitTypeName']

                # Changes (Mothership Core -> Mothership and so on)
                elif event.get('_event', '') ==  'NNet.Replay.Tracker.SUnitTypeChangeEvent' and owned[event['m_unitTagIndex']] and unit == event['m_unitTypeName']:
                   count -= 1
                   if count == 0:
                        result.append(str(event.get('_gameloop', -1)))
                        break

                elif event.get('_event', '') == 'NNet.Replay.Tracker.SUnitDoneEvent':
                    if event['m_unitTagIndex'] in warpInUnits:
                        if warpInUnits[event['m_unitTagIndex']] == unit:                
                            count -= 1
                            if count == 0:
                                result.append(str(event.get('_gameloop', -1)))
                                break

                        del warpInUnits[event['m_unitTagIndex']]

                elif event.get('_event', '') == 'NNet.Replay.Tracker.SUnitDiedEvent':
                    if event['m_unitTagIndex'] in warpInUnits:
                        del warpInUnits[event['m_unitTagIndex']]
                    owned[event['m_unitTagIndex']] = False

            if count > 0:
                result.append(-2)

        return result

    def checkBuildingBenchmarkByPlayerNumber(self,replay,playerNumber,benchmarks):
        result = []

        for benchmark in benchmarks:
            count = []
            arr = benchmark['time'].split(':')
            if len(arr) == 1:
                frames = int(arr[0])*60*16
            else:
                frames = (int(arr[0])*60+int(arr[1]))*16
            building = benchmark['building']
            count = int(benchmark['count'])
            inited = {}
            owned = defaultdict(lambda: False)
            buildings = {}

            for event in replay.trackerEvents:
                if event.get('_event', '') == 'NNet.Replay.Tracker.SUnitBornEvent' and event['m_controlPlayerId'] == playerNumber:
                    born_building = event['m_unitTypeName']
                    if building == born_building:
                        count -= 1
                        if count == 0:
                            result.append(str(event.get('_gameloop', -1)))
                            break
                    owned[event['m_unitTagIndex']] = True

                # Warpins and buildings
                elif event.get('_event', '') == 'NNet.Replay.Tracker.SUnitInitEvent' and event['m_controlPlayerId'] == playerNumber:
                    if building == event['m_unitTypeName']:
                        inited[event['m_unitTagIndex']] = event['m_unitTypeName']
                    else:
                        buildings[event['m_unitTagIndex']] = event['m_unitTypeName']


                # Changes (Mothership Core -> Mothership and so on)
                elif event.get('_event', '') ==  'NNet.Replay.Tracker.SUnitTypeChangeEvent' and owned[event['m_unitTagIndex']] and building == event['m_unitTypeName']:
                   count -= 1
                   if count == 0:
                        result.append(str(event.get('_gameloop', -1)))
                        break

                elif event.get('_event', '') == 'NNet.Replay.Tracker.SUnitDoneEvent':
                    if event['m_unitTagIndex'] in inited:
                        if inited[event['m_unitTagIndex']] == building:                
                            count -= 1
                            if count == 0:
                                result.append(str(event.get('_gameloop', -1)))
                                break

                        del inited[event['m_unitTagIndex']]

                    if event['m_unitTagIndex'] in buildings:
                        if buildings[event['m_unitTagIndex']] == building:                
                            count -= 1
                            if count == 0:
                                result.append(str(event.get('_gameloop', -1)))
                                break
                                
                        del buildings[event['m_unitTagIndex']]

                elif event.get('_event', '') == 'NNet.Replay.Tracker.SUnitDiedEvent':
                    if event['m_unitTagIndex'] in inited:
                        del inited[event['m_unitTagIndex']]
                    owned[event['m_unitTagIndex']] = False

            if count > 0:
                result.append(-2)

        return result

    def checkUpgradeBenchmarkByPlayerNumber(self,replay,playerNumber,benchmarks):
        result = []

        for benchmark in benchmarks:
            arr = benchmark['time'].split(':')
            frames = (int(arr[0])*60+int(arr[1]))*16
            upgrade = benchmark['upgrade']
            upgraded = False

            for event in replay.trackerEvents:
                if event.get('_event', '') == 'NNet.Replay.Tracker.SUpgradeEvent' and event['m_playerId'] == playerNumber and event['m_upgradeTypeName'] == upgrade:
                    result.append(str(event.get('_gameloop', -1)))
                    upgraded=True
                    break

            if not upgraded:
                result.append(-2)

        return result
        
