import os
import spawningtool.parser
import json
import hashlib

class ReplayCollection:
	def __init__(self,replay_dir):
		import re
		import hashlib

		pattern = re.compile(r'.*\.SC2Replay$')

		self.replays = []
		for root, dirs, files in os.walk(replay_dir):
			if files is None:
				continue

			for basename in files:
				try:
					matched = pattern.match(basename)
				except AttributeError:
					matched = fnmatch.fnmatch(basename, pattern)

				if matched:
					replayFullPath = os.path.join(root, basename)
					replay = Replay(replayFullPath)
					if replay.valid:
						self.replays.append(replay)

class Replay:
	def __init__(self,replayfile=None):
		self.metadata = self.getmetadata(replayfile)
		self.replayfile = replayfile
		self.valid = True
		self.buildSequence = {}
		try:
			parsed = spawningtool.parser.parse_replay(replayfile)
		except spawningtool.exception.ReadError:
			print "Could not parse",replayFile,"due to read error"
			self.valid=False
		except spawningtool.exception.ReplayFormatError:
			print "Skipping",replayFile,"due to format error"
			self.valid=False
		except KeyError:
			print "Skipping",replayFile,"KeyError"
			self.valid=False
		except Exception, e:
			print e

		self.buildOrders = self.parse(parsed)
		self.patch = parsed['baseBuild']
		self.allInfo = parsed

		self.hash = hashlib.md5(open(replayfile).read()).hexdigest()

	def parse(self, parsed):
		buildOrders = {}
		try:
			
			eachPlayer = parsed['players']

			for P in eachPlayer:
				player = int(P)
				self.buildSequence[player] = []
				buildOrder = {}
				for element in eachPlayer[player]['buildOrder']:
					name = element['name']
					supply = element['supply']

					buildOrder[supply] = buildOrder.get(supply, {})
					buildOrder[supply][name] = buildOrder[supply].get(name,0)+1
					self.buildSequence[player].append(element)

				buildOrders[player] = buildOrder
		except spawningtool.exception.ReadError:
			print "Could not parse",replayFile,"due to read error"
			self.valid=False
		except spawningtool.exception.ReplayFormatError:
			print "Skipping",replayFile,"due to format error"
			self.valid=False
		except KeyError:
			print "Skipping",replayFile,"KeyError"
			self.valid=False

		return buildOrders

	def getmetadata(self,replayFile):
		metadataFile = os.path.splitext(replayFile)[0] +  ".meta"
		if not os.path.exists(metadataFile):
			return {}
		else:
			return json.load(open(metadataFile))

class ReplayLabel:
	@staticmethod
	def create_label(replay,player):
		label = replay.metadata[player]["buildOrderLabel"]
		repHash = replay.hash #[0:5]
		return "\"B_" + str(label.replace("-","_")) + "_" + str(player) + "_" + str(repHash)[0:4] + "\""