from collections import defaultdict
import json
import os
from s2protocol.s2protocol import *
from s2protocol.protocol26490 import *


replay_folder = "C:/Users/Christian/Documents/StarCraft II/Accounts/486076/2-S2-1-891100/Replays/Multiplayer"
filelist = os.listdir(replay_folder)
newest = max(filelist, key=lambda x: os.stat(os.path.join(replay_folder,x)).st_mtime)
print os.path.join(replay_folder,newest)

benchmarks = []
benchmark_names = []

for LOAD in ['data/tvp-bombers-reaper-fe.json', 'data/Banshee_Opening.json', 'data/tvz-flashs-reaper-fe-into-reaperhellion.json']:
	loaded = json.load(open(LOAD))
	for benchmark in loaded["Benchmarks"]:
		benchmarks.append(benchmark)
		benchmark_names.append(loaded["BuildName"])

replay_file = os.path.join(replay_folder,newest)

archive = mpyq.MPQArchive(replay_file)
	
logger = EventLogger()

# Read the protocol header, this can be read with any protocol
contents = archive.header['user_data_header']['content']
header = protocol15405.decode_replay_header(contents)

# The header's baseBuild determines which protocol to use
baseBuild = header['m_version']['m_baseBuild']


for player in [1,2]:
	benchmark_satisfied = [False]*len(benchmarks)

	units = defaultdict(int)
	upgrades = defaultdict(bool)

	contents = archive.read_file('replay.tracker.events')
	for event in decode_replay_tracker_events(contents):

		if event['_event'] == 'NNet.Replay.Tracker.SUnitBornEvent':
			if event['m_controlPlayerId'] != player:
				continue

			frame = event['_gameloop']/16
			seconds = frame%60
			minutes = frame/60
			name = event['m_unitTypeName']
			units[name]+=1
		elif event['_event'] == 'NNet.Replay.Tracker.SUpgradeEvent':
			if event['m_playerId'] != player:
				continue

			frame = event['_gameloop']/16
			seconds = frame%60
			minutes = frame/60
			name = event['m_upgradeTypeName']
			upgrades[name] = True

		for i in range(len(benchmarks)):
			if benchmark_satisfied[i]:
				continue

			satisfied=True

			for unit in benchmarks[i]['Units']:
				for key in unit:
					if units[key] < unit[key]:
						satisfied=False

			for upgrade in benchmarks[i]['Upgrades']:
				if not upgrades[upgrade]:
					satisfied=False

			if satisfied:
				print "Benchmark",i,benchmark_names[i],"SATISFIED for player",str(player) + " at " + str(minutes) + ":" +  str(seconds) + " (goal",benchmarks[i]['Time'] + ")"
				benchmark_satisfied[i] = True


	for i in range(len(benchmarks)):
		if benchmark_satisfied[i]:
			continue

		print "Benchmark",i,benchmark_names[i],"not satisfied for player",str(player) + " (goal",benchmarks[i]['Time'] + ")"

#		for unit in benchmarks[i]['Units']:
#			for key in unit:
#				if units[key] < unit[key]:
#					satisfied=False
#
#		for upgrade in benchmarks[i]['Upgrades']:
#			if not upgrades[upgrade]:
#				satisfied=False
#